@@ -5,10 +5,12 @@
 
 #include "htmlutils.h"
 #include "objecttreeparser.h"
+#include "utils.h"
 
 #include <KLocalizedString>
 
 #include <QDebug>
+#include <QGpgME/Protocol>
 #include <QRegularExpression>
 #include <QStringLiteral>
 #include <QTextDocument>
@@ -174,13 +176,16 @@ public:
         MimeTreeParser::MessagePart::List filteredParts;
 
         for (const auto &part : parts) {
-            const auto contentType = part->node()->contentType();
-            if (contentType && contentType->hasParameter(QStringLiteral("protected-headers"))) {
-                const auto contentDisposition = part->node()->contentDisposition();
-                if (contentDisposition && contentDisposition->disposition() == KMime::Headers::CDinline) {
-                    continue;
+            if (part->node()) {
+                const auto contentType = part->node()->contentType();
+                if (contentType && contentType->hasParameter(QStringLiteral("protected-headers"))) {
+                    const auto contentDisposition = part->node()->contentDisposition();
+                    if (contentDisposition && contentDisposition->disposition() == KMime::Headers::CDinline) {
+                        continue;
+                    }
                 }
             }
+
             filteredParts << part;
         }
 
@@ -500,8 +505,22 @@ QVariant PartModel::data(const QModelIndex &index, int role) const
             return messagePart->error();
         case ErrorString: {
             switch (messagePart->error()) {
-            case MimeTreeParser::MessagePart::NoKeyError:
-                return i18ndc("mimetreeparser", "@info:status", "No key available.");
+            case MimeTreeParser::MessagePart::NoKeyError: {
+                if (auto encryptedMessagePart = dynamic_cast<MimeTreeParser::EncryptedMessagePart *>(messagePart)) {
+                    if (encryptedMessagePart->isNoSecKey()) {
+                        QString errorMessage = i18ndc("mimetreeparser", "@info:status", "No secret key found to decrypt the message.");
+                        if (!encryptedMessagePart->decryptRecipients().empty()) {
+                            errorMessage += QLatin1Char(' ') + i18ndc("mimetreeparser", "@info:status", "The message is encrypted for the following keys:");
+                            errorMessage +=
+                                MimeTreeParser::decryptRecipientsToHtml(encryptedMessagePart->decryptRecipients(), encryptedMessagePart->cryptoProto());
+                        }
+                        return errorMessage;
+                    }
+                }
+            }
+
+                return messagePart->errorString();
+
             case MimeTreeParser::MessagePart::PassphraseError:
                 return i18ndc("mimetreeparser", "@info:status", "Wrong passphrase.");
             case MimeTreeParser::MessagePart::UnknownError:
