@@ -212,25 +212,33 @@ void MessageViewerDialog::Private::printInternal(QPrinter *printer)
     messageViewer->print(&painter, pageRect.width());
 }
 
+MessageViewerDialog::MessageViewerDialog(const QList<KMime::Message::Ptr> &messages, QWidget *parent)
+    : QDialog(parent)
+    , d(std::make_unique<Private>())
+{
+    d->messages += messages;
+    initGUI();
+}
+
 MessageViewerDialog::MessageViewerDialog(const QString &fileName, QWidget *parent)
     : QDialog(parent)
     , d(std::make_unique<Private>())
 {
     d->fileName = fileName;
+    d->messages += MimeTreeParser::Core::FileOpener::openFile(fileName);
+    initGUI();
+}
+
+void MessageViewerDialog::initGUI()
+{
     const auto mainLayout = new QVBoxLayout(this);
     mainLayout->setContentsMargins({});
 
     const auto layout = new QVBoxLayout;
-    layout->setContentsMargins(style()->pixelMetric(QStyle::PM_LayoutLeftMargin, nullptr, this),
-                               style()->pixelMetric(QStyle::PM_LayoutTopMargin, nullptr, this),
-                               style()->pixelMetric(QStyle::PM_LayoutRightMargin, nullptr, this),
-                               style()->pixelMetric(QStyle::PM_LayoutBottomMargin, nullptr, this));
 
     const auto menuBar = d->createMenuBar(this);
     mainLayout->setMenuBar(menuBar);
 
-    d->messages += MimeTreeParser::Core::FileOpener::openFile(fileName);
-
     if (d->messages.isEmpty()) {
         auto errorMessage = new KMessageWidget(this);
         errorMessage->setMessageType(KMessageWidget::Error);
@@ -275,6 +283,10 @@ MessageViewerDialog::MessageViewerDialog(const QString &fileName, QWidget *paren
     layout->addWidget(d->messageViewer);
 
     auto buttonBox = new QDialogButtonBox(this);
+    buttonBox->setContentsMargins(style()->pixelMetric(QStyle::PM_LayoutLeftMargin, nullptr, this),
+                                  style()->pixelMetric(QStyle::PM_LayoutTopMargin, nullptr, this),
+                                  style()->pixelMetric(QStyle::PM_LayoutRightMargin, nullptr, this),
+                                  style()->pixelMetric(QStyle::PM_LayoutBottomMargin, nullptr, this));
     auto closeButton = buttonBox->addButton(QDialogButtonBox::Close);
     connect(closeButton, &QPushButton::pressed, this, &QDialog::accept);
     layout->addWidget(buttonBox);
@@ -286,3 +298,5 @@ QList<KMime::Message::Ptr> MessageViewerDialog::messages() const
 {
     return d->messages;
 }
+
+#include "moc_messageviewerdialog.cpp"
